@@ -41,7 +41,6 @@ enum {
   PAD_REMOVED,
   ERROR,
   EOS,
-  DEEP_NOTIFY,
   LAST_SIGNAL
 };
 
@@ -60,7 +59,6 @@ static void			gst_element_real_set_property	(GObject *object, guint prop_id,
 								 const GValue *value, GParamSpec *pspec);
 static void			gst_element_real_get_property	(GObject *object, guint prop_id, GValue *value, 
 								 GParamSpec *pspec);
-static void			gst_element_dispatch_properties_changed (GObject * object, guint n_pspecs, GParamSpec **pspecs);
 
 static void 			gst_element_dispose 		(GObject *object);
 
@@ -137,21 +135,10 @@ gst_element_class_init (GstElementClass *klass)
     g_signal_new ("eos", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GstElementClass,eos), NULL, NULL,
                   gst_marshal_VOID__VOID, G_TYPE_NONE, 0);
-  gst_element_signals[DEEP_NOTIFY] =
-    g_signal_new ("deep_notify", G_TYPE_FROM_CLASS (klass), 
-		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED | G_SIGNAL_NO_HOOKS,
-		  G_STRUCT_OFFSET (GstElementClass, deep_notify), NULL, NULL,
-		  gst_marshal_VOID__OBJECT_PARAM, G_TYPE_NONE,
-		  2, G_TYPE_OBJECT, G_TYPE_PARAM);
-
 
   gobject_class->set_property 		= GST_DEBUG_FUNCPTR (gst_element_real_set_property);
   gobject_class->get_property 		= GST_DEBUG_FUNCPTR (gst_element_real_get_property);
 
-  /* see the comments at gst_element_dispatch_properties_changed */
-  gobject_class->dispatch_properties_changed
-    = GST_DEBUG_FUNCPTR (gst_element_dispatch_properties_changed);
-
   gobject_class->dispose 		= GST_DEBUG_FUNCPTR (gst_element_dispose);
 
 #ifndef GST_DISABLE_LOADSAVE
@@ -214,87 +201,6 @@ gst_element_real_get_property (GObject *object, guint prop_id, GValue *value, GP
     (oclass->get_property) (object, prop_id, value, pspec);
 }
 
-/* Changing a GObject property of an element will result in "deep_notify"
- * signals being emitted by the element itself, as well as in each parent
- * element. This is so that an application can connect a listener to the
- * top-level bin to catch property-change notifications for all contained
- * elements. */
-static void
-gst_element_dispatch_properties_changed (GObject     *object,
-                                         guint        n_pspecs,
-                                         GParamSpec **pspecs)
-{
-  GstObject *gst_object;
-  guint i;
-
-  /* do the standard dispatching */
-  G_OBJECT_CLASS (parent_class)->dispatch_properties_changed (object, n_pspecs, pspecs);
-
-  /* now let the parent dispatch those, too */
-  gst_object = GST_OBJECT_PARENT (object);
-  while (gst_object) {
-    /* need own category? */
-    for (i = 0; i < n_pspecs; i++) {
-      GST_DEBUG (GST_CAT_EVENT, "deep notification from %s to %s (%s)", GST_OBJECT_NAME (object), 
-		    GST_OBJECT_NAME (gst_object), pspecs[i]->name);
-      g_signal_emit (gst_object, gst_element_signals[DEEP_NOTIFY], g_quark_from_string (pspecs[i]->name), 
-		     (GstObject *) object, pspecs[i]);
-    }
-
-    gst_object = GST_OBJECT_PARENT (gst_object);
-  }
-}
-
-/** 
- * gst_element_default_deep_notify:
- * @object: the #GObject that signalled the notify.
- * @orig: a #GstObject that initiated the notify.
- * @pspec: a #GParamSpec of the property.
- * @excluded_props: a set of user-specified properties to exclude.
- *
- * Adds a default deep_notify signal callback to an
- * element. The user data should contain a pointer to an array of
- * strings that should be excluded from the notify.
- * The default handler will print the new value of the property 
- * using g_print.
- */
-void
-gst_element_default_deep_notify (GObject *object, GstObject *orig, 
-                                 GParamSpec *pspec, gchar **excluded_props)
-{
-  GValue value = { 0, }; /* the important thing is that value.type = 0 */
-  gchar *str = 0;
-
-  if (pspec->flags & G_PARAM_READABLE) {
-    /* let's not print these out for excluded properties... */
-    while (excluded_props != NULL && *excluded_props != NULL) {
-      if (strcmp (pspec->name, *excluded_props) == 0)
-        return;
-      excluded_props++;
-    }
-    g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-    g_object_get_property (G_OBJECT (orig), pspec->name, &value);
-
-    if (G_IS_PARAM_SPEC_ENUM (pspec)) {
-      GEnumValue *enum_value;
-      enum_value = g_enum_get_value (G_ENUM_CLASS (g_type_class_ref (pspec->value_type)),
-      g_value_get_enum (&value));
-
-      str = g_strdup_printf ("%s (%d)", enum_value->value_nick, 
-	                                enum_value->value);
-    }
-    else {
-      str = g_strdup_value_contents (&value);
-    }
-    g_print ("%s: %s = %s\n", GST_OBJECT_NAME (orig), pspec->name, str);
-    g_free (str);
-    g_value_unset (&value);
-  } else {
-    g_warning ("Parameter %s not readable in %s.", 
-	       pspec->name, GST_OBJECT_NAME (orig));
-  }
-}
-
 /** 
  * gst_element_default_error:
  * @object: a #GObject that signalled the error.
@@ -310,7 +216,9 @@ gst_element_default_deep_notify (GObject *object, GstObject *orig,
 void
 gst_element_default_error (GObject *object, GstObject *orig, gchar *error)
 { 
-  g_print ("ERROR: %s: %s\n", GST_OBJECT_NAME (orig), error);
+  gchar *name = gst_object_get_path_string (orig);
+  g_print ("ERROR: %s: %s\n", name, error);
+  g_free (name);
 } 
 
 typedef struct {
@@ -761,76 +669,36 @@ gst_element_release_request_pad (GstElement *element, GstPad *pad)
     (oclass->release_pad) (element, pad);
 }
 
-
 /**
- * gst_element_set_name:
- * @element: a #GstElement to set the name of.
- * @name: the new name of the element.
+ * gst_element_requires_clock:
+ * @element: a #GstElement to query
  *
- * Sets the name of the element, getting rid of the old name if there was
- * one.
+ * Query if the element requiresd a clock
+ *
+ * Returns: TRUE if the element requires a clock
  */
-void
-gst_element_set_name (GstElement *element, const gchar *name)
+gboolean
+gst_element_requires_clock (GstElement *element)
 {
-  g_return_if_fail (element != NULL);
-  g_return_if_fail (GST_IS_ELEMENT (element));
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
-  gst_object_set_name (GST_OBJECT (element), name);
+  return (CLASS (element)->set_clock != NULL);
 }
 
 /**
- * gst_element_get_name:
- * @element: a #GstElement to get the name of.
+ * gst_element_provides_clock:
+ * @element: a #GstElement to query
  *
- * Gets the name of the element.
+ * Query if the element provides a clock
  *
- * Returns: the name of the element.
+ * Returns: TRUE if the element provides a clock
  */
-const gchar*
-gst_element_get_name (GstElement *element)
+gboolean
+gst_element_provides_clock (GstElement *element)
 {
-  g_return_val_if_fail (element != NULL, NULL);
-  g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
-  return GST_OBJECT_NAME (element);
-}
-
-/**
- * gst_element_set_parent:
- * @element: a #GstElement to set parent of.
- * @parent: the new #GstObject parent of the object.
- *
- * Sets the parent of the element.
- */
-void
-gst_element_set_parent (GstElement *element, GstObject *parent)
-{
-  g_return_if_fail (element != NULL);
-  g_return_if_fail (GST_IS_ELEMENT (element));
-  g_return_if_fail (GST_OBJECT_PARENT (element) == NULL);
-  g_return_if_fail (parent != NULL);
-  g_return_if_fail (GST_IS_OBJECT (parent));
-  g_return_if_fail ((gpointer)element != (gpointer)parent);
-
-  gst_object_set_parent (GST_OBJECT (element), parent);
-}
-
-/**
- * gst_element_get_parent:
- * @element: a #GstElement to get the parent of.
- *
- * Gets the parent of the element.
- *
- * Returns: the #GstObject parent of the element.
- */
-GstObject*
-gst_element_get_parent (GstElement *element)
-{
-  g_return_val_if_fail (element != NULL, NULL);
-  g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
-
-  return GST_OBJECT_PARENT (element);
+  return (CLASS (element)->get_clock != NULL);
 }
 
 /**
@@ -843,11 +711,10 @@ gst_element_get_parent (GstElement *element)
 void
 gst_element_set_clock (GstElement *element, GstClock *clock)
 {
-  g_return_if_fail (element != NULL);
   g_return_if_fail (GST_IS_ELEMENT (element));
 
-  if (element->setclockfunc)
-    element->setclockfunc (element, clock);
+  if (CLASS (element)->set_clock)
+    CLASS (element)->set_clock (element, clock);
 
   element->clock = clock;
 }
@@ -866,8 +733,8 @@ gst_element_get_clock (GstElement *element)
   g_return_val_if_fail (element != NULL, NULL);
   g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
   
-  if (element->getclockfunc)
-    return element->getclockfunc (element);
+  if (CLASS (element)->get_clock)
+    return CLASS (element)->get_clock (element);
 
   return NULL;
 }
@@ -888,7 +755,6 @@ gst_element_clock_wait (GstElement *element, GstClock *clock, GstClockTime time,
 {
   GstClockReturn res;
 
-  g_return_val_if_fail (element != NULL, GST_CLOCK_ERROR);
   g_return_val_if_fail (GST_IS_ELEMENT (element), GST_CLOCK_ERROR);
 
   if (GST_ELEMENT_SCHED (element)) {
@@ -900,6 +766,35 @@ gst_element_clock_wait (GstElement *element, GstClock *clock, GstClockTime time,
   return res;
 }
 
+gboolean
+gst_element_is_cachable (GstElement *element)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+
+  return (CLASS (element)->set_cache != NULL);
+}
+
+void
+gst_element_set_cache (GstElement *element, GstCache *cache)
+{
+  g_return_if_fail (GST_IS_ELEMENT (element));
+  g_return_if_fail (GST_IS_CACHE (cache));
+
+  if (CLASS (element)->set_cache)
+    CLASS (element)->set_cache (element, cache);
+}
+
+GstCache*
+gst_element_get_cache (GstElement *element)
+{
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
+
+  if (CLASS (element)->get_cache)
+    return CLASS (element)->get_cache (element);
+
+  return NULL;
+}
+
 
 /**
  * gst_element_release_locks:
@@ -1085,11 +980,10 @@ gst_element_get_pad (GstElement *element, const gchar *name)
   g_return_val_if_fail (GST_IS_ELEMENT (element), NULL);
   g_return_val_if_fail (name != NULL, NULL);
 
-  if ((pad = gst_element_get_static_pad (element, name)))
-    return pad;
-  
-  pad = gst_element_get_request_pad (element, name);
-  
+  pad = gst_element_get_static_pad (element, name);
+  if (!pad) 
+    pad = gst_element_get_request_pad (element, name);
+
   return pad;
 }
 
@@ -2201,6 +2095,13 @@ gst_element_change_state (GstElement *element)
       break;
   }
 
+  parent = GST_ELEMENT_PARENT (element);
+
+  GST_DEBUG_ELEMENT (GST_CAT_STATES, element,
+                     "signaling state change from %s to %s",
+                     gst_element_state_get_name (old_state),
+                     gst_element_state_get_name (GST_STATE (element)));
+
   /* tell the scheduler if we have one */
   if (element->sched) {
     if (gst_scheduler_state_transition (element->sched, element, 
@@ -2209,12 +2110,6 @@ gst_element_change_state (GstElement *element)
     }
   }
 
-  parent = GST_ELEMENT_PARENT (element);
-
-  GST_DEBUG_ELEMENT (GST_CAT_STATES, element,
-                     "signaling state change from %s to %s",
-                     gst_element_state_get_name (old_state),
-                     gst_element_state_get_name (GST_STATE (element)));
   g_signal_emit (G_OBJECT (element), gst_element_signals[STATE_CHANGE],
 		 0, old_state, GST_STATE (element));
 
@@ -2366,7 +2261,8 @@ gst_element_save_thyself (GstObject *object,
       else if (G_IS_PARAM_SPEC_ENUM (spec))
 	contents = g_strdup_printf ("%d", g_value_get_enum (&value));
       else if (G_IS_PARAM_SPEC_INT64 (spec))
-	contents = g_strdup_printf ("%lld", g_value_get_int64 (&value));
+	contents = g_strdup_printf ("%" G_GINT64_FORMAT,
+			            g_value_get_int64 (&value));
       else
 	contents = g_strdup_value_contents (&value);
       
