@@ -94,27 +94,6 @@ gst_scheduler_setup (GstScheduler *sched)
     CLASS (sched)->setup (sched);
 }
 
-/**
- * gst_scheduler_get_preferred_stack:
- * @sched: a #GstScheduler to query.
- * @stack: the pointer to store the location of the preferred stack in.
- * @size: the pointer to store the size of the preferred stack in.
- *
- * Gets the preferred stack location and size of this scheduler.
- *
- * Returns: TRUE if the scheduler suggested a preferred stacksize and location.
- */
-gboolean
-gst_scheduler_get_preferred_stack (GstScheduler *sched, gpointer *stack, gulong *size)
-{
-  g_return_val_if_fail (GST_IS_SCHEDULER (sched), FALSE);
-
-  if (CLASS (sched)->get_preferred_stack)
-    return CLASS (sched)->get_preferred_stack (sched, stack, size);
-  
-  return FALSE;
-}
-
 /**
  * gst_scheduler_reset:
  * @sched: a #GstScheduler to reset.
@@ -212,11 +191,11 @@ gst_scheduler_add_element (GstScheduler *sched, GstElement *element)
   /* if it's not inside this scheduler, it has to be NULL */
   g_assert (GST_ELEMENT_SCHED (element) == NULL);
 
-  if (element->getclockfunc) {
+  if (gst_element_provides_clock (element)) {
     sched->clock_providers = g_list_prepend (sched->clock_providers, element);
     GST_DEBUG (GST_CAT_CLOCK, "added clock provider %s", GST_ELEMENT_NAME (element));
   }
-  if (element->setclockfunc) {
+  if (gst_element_requires_clock (element)) {
     sched->clock_receivers = g_list_prepend (sched->clock_receivers, element);
     GST_DEBUG (GST_CAT_CLOCK, "added clock receiver %s", GST_ELEMENT_NAME (element));
   }
@@ -566,6 +545,7 @@ gst_scheduler_set_clock (GstScheduler *sched, GstClock *clock)
 
     GST_DEBUG (GST_CAT_CLOCK, "scheduler setting clock %p (%s) on element %s", clock, 
 		(clock ? GST_OBJECT_NAME (clock) : "nil"), GST_ELEMENT_NAME (element));
+    
     gst_element_set_clock (element, clock);
     receivers = g_list_next (receivers);
   }
@@ -618,7 +598,11 @@ gst_scheduler_clock_wait (GstScheduler *sched, GstElement *element, GstClock *cl
   if (CLASS (sched)->clock_wait)
     return CLASS (sched)->clock_wait (sched, element, clock, time, jitter);
   else
-    return gst_clock_wait (clock, time, jitter);
+  {
+    GstClockID id = gst_clock_new_single_shot_id (clock, time);
+
+    return gst_clock_id_wait (id, jitter);
+  }
 
   return GST_CLOCK_TIMEOUT;
 }
