@@ -30,6 +30,10 @@
 #include "gstscheduler.h"
 #include "gstqueue.h"
 
+#define STACK_SIZE 0x200000
+
+#define g_thread_equal(a,b) ((a) == (b))
+
 GstElementDetails gst_thread_details = {
   "Threaded container",
   "Generic/Bin",
@@ -62,31 +66,35 @@ enum {
 
 
 
-static void			gst_thread_class_init		(GstThreadClass *klass);
-static void			gst_thread_init			(GstThread *thread);
+static void		gst_thread_class_init		(GstThreadClass *klass);
+static void		gst_thread_init			(GstThread *thread);
 
-static void 			gst_thread_dispose 	(GObject *object);
+static void		gst_thread_dispose		(GObject *object);
 
-static void			gst_thread_set_property		(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
-static void			gst_thread_get_property		(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
+static void		gst_thread_set_property		(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
+static void		gst_thread_get_property		(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static GstElementStateReturn	gst_thread_change_state		(GstElement *element);
+static
+GstElementStateReturn	gst_thread_change_state		(GstElement *element);
 
 #ifndef GST_DISABLE_LOADSAVE
-static xmlNodePtr		gst_thread_save_thyself		(GstObject *object, xmlNodePtr parent);
-static void			gst_thread_restore_thyself	(GstObject *object, xmlNodePtr self);
+static xmlNodePtr	gst_thread_save_thyself		(GstObject *object,
+		                                         xmlNodePtr parent);
+static void		gst_thread_restore_thyself	(GstObject *object,
+		                                         xmlNodePtr self);
 #endif
 
-static void*			gst_thread_main_loop		(void *arg);
+static void*		gst_thread_main_loop		(void *arg);
 
 #define GST_TYPE_THREAD_SCHEDPOLICY (gst_thread_schedpolicy_get_type())
 static GType
 gst_thread_schedpolicy_get_type(void) {
   static GType thread_schedpolicy_type = 0;
   static GEnumValue thread_schedpolicy[] = {
-    {SCHED_OTHER, "SCHED_OTHER", "Normal Scheduling"},
-    {SCHED_FIFO,  "SCHED_FIFO",  "FIFO Scheduling (requires root)"},
-    {SCHED_RR,    "SCHED_RR",    "Round-Robin Scheduling (requires root)"},
+    {G_THREAD_PRIORITY_LOW,    "LOW", "Low Priority Scheduling"},
+    {G_THREAD_PRIORITY_NORMAL, "NORMAL",  "Normal Scheduling"},
+    {G_THREAD_PRIORITY_HIGH,   "HIGH",  "High Priority Scheduling"},
+    {G_THREAD_PRIORITY_URGENT, "URGENT", "Urgent Scheduling"},
     {0, NULL, NULL},
   };
   if (!thread_schedpolicy_type) {
@@ -96,7 +104,7 @@ gst_thread_schedpolicy_get_type(void) {
 }
 
 static GstBinClass *parent_class = NULL;
-static guint gst_thread_signals[LAST_SIGNAL] = { 0 }; 
+static guint gst_thread_signals[LAST_SIGNAL] = { 0 };
 
 GType
 gst_thread_get_type(void) {
@@ -104,18 +112,15 @@ gst_thread_get_type(void) {
 
   if (!thread_type) {
     static const GTypeInfo thread_info = {
-      sizeof(GstThreadClass),
-      NULL,
-      NULL,
-      (GClassInitFunc)gst_thread_class_init,
-      NULL,
-      NULL,
-      sizeof(GstThread),
+      sizeof (GstThreadClass), NULL, NULL,
+      (GClassInitFunc) gst_thread_class_init, NULL, NULL,
+      sizeof (GstThread),
       4,
-      (GInstanceInitFunc)gst_thread_init,
+      (GInstanceInitFunc) gst_thread_init,
       NULL
     };
-    thread_type = g_type_register_static(GST_TYPE_BIN, "GstThread", &thread_info, 0);
+    thread_type = g_type_register_static (GST_TYPE_BIN, "GstThread",
+		                          &thread_info, 0);
   }
   return thread_type;
 }
@@ -137,7 +142,7 @@ gst_thread_class_init (GstThreadClass *klass)
 
   g_object_class_install_property(G_OBJECT_CLASS (klass), ARG_SCHEDPOLICY,
     g_param_spec_enum("schedpolicy", "Scheduling Policy", "The scheduling policy of the thread",
-                      GST_TYPE_THREAD_SCHEDPOLICY, SCHED_OTHER, G_PARAM_READWRITE));
+                      GST_TYPE_THREAD_SCHEDPOLICY, G_THREAD_PRIORITY_NORMAL, G_PARAM_READWRITE));
   g_object_class_install_property(G_OBJECT_CLASS (klass), ARG_PRIORITY,
     g_param_spec_int("priority", "Scheduling Priority", "The scheduling priority of the thread",
                      0, 99, 0, G_PARAM_READWRITE));
@@ -174,13 +179,14 @@ gst_thread_init (GstThread *thread)
   GST_FLAG_SET (thread, GST_BIN_SELF_SCHEDULABLE);
 
   scheduler = gst_scheduler_factory_make (NULL, GST_ELEMENT (thread));
+  g_assert (scheduler);
 
   thread->lock = g_mutex_new ();
   thread->cond = g_cond_new ();
 
   thread->ppid = getpid ();
-  thread->thread_id = (pthread_t) -1;
-  thread->sched_policy = SCHED_OTHER;
+  thread->thread_id = (GThread *) NULL; /* set in NULL -> READY */
+  thread->sched_policy = G_THREAD_PRIORITY_NORMAL;
   thread->priority = 0;
   thread->stack = NULL;
 }
@@ -192,6 +198,14 @@ gst_thread_dispose (GObject *object)
 
   GST_DEBUG (GST_CAT_REFCOUNTING, "dispose");
 
+  g_mutex_lock (thread->lock);
+  if (GST_STATE (thread) != GST_STATE_NULL) {
+    GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
+  }
+  g_mutex_unlock (thread->lock);
+
+  gst_element_set_state (GST_ELEMENT (thread), GST_STATE_NULL);
+
   g_mutex_free (thread->lock);
   g_cond_free (thread->cond);
 
@@ -268,7 +282,7 @@ gst_thread_new (const gchar *name)
 #define THR_INFO(format,args...) \
   GST_INFO_ELEMENT(GST_CAT_THREAD, thread, "sync(" GST_DEBUG_THREAD_FORMAT "): " format , \
   GST_DEBUG_THREAD_ARGS(thread->pid) , ## args )
-  
+
 #define THR_DEBUG(format,args...) \
   GST_DEBUG_ELEMENT(GST_CAT_THREAD, thread, "sync(" GST_DEBUG_THREAD_FORMAT "): " format , \
   GST_DEBUG_THREAD_ARGS(thread->pid) , ## args )
@@ -285,7 +299,7 @@ gst_thread_new (const gchar *name)
   GST_DEBUG_ELEMENT(GST_CAT_THREAD, thread, "sync-main(" GST_DEBUG_THREAD_FORMAT "): " format , \
   GST_DEBUG_THREAD_ARGS(thread->ppid) , ## args )
 
-static GstElementStateReturn 
+static GstElementStateReturn
 gst_thread_update_state (GstThread *thread)
 {
   GST_DEBUG_ELEMENT (GST_CAT_THREAD, thread, "updating state of thread");
@@ -304,13 +318,13 @@ gst_thread_update_state (GstThread *thread)
 
 
 static GstElementStateReturn
-gst_thread_change_state (GstElement * element)
+gst_thread_change_state (GstElement *element)
 {
   GstThread *thread;
   gboolean stateset = GST_STATE_SUCCESS;
   gint transition;
-  pthread_t self = pthread_self ();
-  glong stacksize;
+  GThread *self = g_thread_self ();
+  GError *error = NULL;
 
   g_return_val_if_fail (GST_IS_THREAD (element), GST_STATE_FAILURE);
   g_return_val_if_fail (gst_has_threads (), GST_STATE_FAILURE);
@@ -323,9 +337,10 @@ gst_thread_change_state (GstElement * element)
 	    gst_element_state_get_name (GST_STATE (element)),
 	    gst_element_state_get_name (GST_STATE_PENDING (element)));
 
-  if (pthread_equal (self, thread->thread_id)) {
+  if (g_thread_equal (self, thread->thread_id)) {
     GST_DEBUG (GST_CAT_THREAD,
-	       "no sync(" GST_DEBUG_THREAD_FORMAT "): setting own thread's state to spinning",
+	       "no sync(" GST_DEBUG_THREAD_FORMAT "): "
+	       "setting own thread's state to spinning",
 	       GST_DEBUG_THREAD_ARGS (thread->pid));
     return gst_thread_update_state (thread);
   }
@@ -337,70 +352,31 @@ gst_thread_change_state (GstElement * element)
 
       THR_DEBUG ("creating thread \"%s\"", GST_ELEMENT_NAME (element));
 
-      /* this bit of code handles creation of pthreads
+      /* this bit of code handles creation of GThreads
        * this is therefor tricky code
        * compare it with the block of code that handles the destruction
        * in GST_STATE_READY_TO_NULL below
        */
       g_mutex_lock (thread->lock);
 
-      /* create attribute struct for pthread
-       * and assign stack pointer and size to it
-       *
-       * the default state of a pthread is PTHREAD_CREATE_JOINABLE
-       * (see man pthread_attr_init)
-       * - other thread can sync on termination
-       * - thread resources are kept allocated until other thread performs
-       *   pthread_join
-       */
-
-      if (pthread_attr_init (&thread->attr) != 0)
-	g_warning ("pthread_attr_init returned an error !");
-
-      /* this function should return a newly allocated stack
-       * (using whatever method)
-       * which we can initiate the pthreads with
-       * the stack should be freed in 
-       */
-      if (gst_scheduler_get_preferred_stack (GST_ELEMENT_SCHED (element), 
-	                                     &thread->stack, &stacksize)) {
-#ifdef HAVE_PTHREAD_ATTR_SETSTACK
-        if (pthread_attr_setstack (&thread->attr, 
-	                           thread->stack, stacksize) != 0) {
-          g_warning ("pthread_attr_setstack failed\n");
-          return GST_STATE_FAILURE;
-        }
-#else
-        if (pthread_attr_setstackaddr (&thread->attr, thread->stack) != 0) {
-	  g_warning ("pthread_attr_setstackaddr failed\n");
-	  return GST_STATE_FAILURE;
-	}
-        if (pthread_attr_setstacksize (&thread->attr, stacksize) != 0) {
-	  g_warning ("pthread_attr_setstacksize failed\n");
-	  return GST_STATE_FAILURE;
-	}
-#endif
-	GST_DEBUG (GST_CAT_THREAD, "pthread attr set stack at %p of size %ld", 
-		   thread->stack, stacksize);
-      }
-      else {
-	g_warning ("scheduler did not return a preferred stack");
-      }
-
-      /* create a new pthread
+      /* create a new GThread
        * use the specified attributes
        * make it execute gst_thread_main_loop (thread)
        */
-      GST_DEBUG (GST_CAT_THREAD, "going to pthread_create...");
-      if (pthread_create (&thread->thread_id, &thread->attr, 
-	  gst_thread_main_loop, thread) != 0) {
-        GST_DEBUG (GST_CAT_THREAD, "pthread_create failed");
+      GST_DEBUG (GST_CAT_THREAD, "going to g_thread_create_full...");
+      thread->thread_id = g_thread_create_full(gst_thread_main_loop,
+	  thread, STACK_SIZE, TRUE, TRUE, G_THREAD_PRIORITY_NORMAL,
+	  &error);
+
+      if (!thread->thread_id){
+        GST_DEBUG (GST_CAT_THREAD, "g_thread_create_full failed");
 	g_mutex_unlock (thread->lock);
-        GST_DEBUG (GST_CAT_THREAD, "could not create thread \"%s\"", 
+        GST_DEBUG (GST_CAT_THREAD, "could not create thread \"%s\"",
 	           GST_ELEMENT_NAME (element));
+
 	return GST_STATE_FAILURE;
       }
-      GST_DEBUG (GST_CAT_THREAD, "pthread created");
+      GST_DEBUG (GST_CAT_THREAD, "GThread created");
 
       /* wait for it to 'spin up' */
       THR_DEBUG ("waiting for child thread spinup");
@@ -426,7 +402,7 @@ gst_thread_change_state (GstElement * element)
         gst_element_enable_threadsafe_properties ((GstElement*)elements->data);
         elements = g_list_next (elements);
       }
-      
+
       THR_DEBUG ("telling thread to start spinning");
       g_mutex_lock (thread->lock);
       THR_DEBUG ("signaling");
@@ -435,6 +411,11 @@ gst_thread_change_state (GstElement * element)
       g_cond_wait (thread->cond, thread->lock);
       THR_DEBUG ("got ack");
       g_mutex_unlock (thread->lock);
+
+      /* we're actually doing async notification, the state could
+       * have changed already (eg. EOS) here. commented out as
+       * it needs more thinking... */
+      /* stateset = GST_STATE_ASYNC; */
       break;
     }
     case GST_STATE_PLAYING_TO_PAUSED:
@@ -448,7 +429,7 @@ gst_thread_change_state (GstElement * element)
        * change_state()).
        * + the pending state was already set by gstelement.c::set_state()
        * + unlock all elements so the bottom half can start the state change.
-       */ 
+       */
       g_mutex_lock (thread->lock);
 
       GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
@@ -456,14 +437,14 @@ gst_thread_change_state (GstElement * element)
       while (elements) {
 	GstElement *element = GST_ELEMENT (elements->data);
 	GList *pads;
-	
 
 	g_assert (element);
 	THR_DEBUG ("  waking element \"%s\"", GST_ELEMENT_NAME (element));
 	elements = g_list_next (elements);
 
 	if (!gst_element_release_locks (element)) {
-          g_warning ("element %s could not release locks", GST_ELEMENT_NAME (element));
+          g_warning ("element %s could not release locks",
+		     GST_ELEMENT_NAME (element));
 	}
 
 	pads = GST_ELEMENT_PADS (element);
@@ -523,6 +504,7 @@ gst_thread_change_state (GstElement * element)
       g_cond_wait (thread->cond, thread->lock);
       THR_DEBUG ("got ack");
 
+
       /* this block of code is very tricky
        * basically, we try to clean up the whole thread and
        * everything related to it in the right order without
@@ -530,26 +512,11 @@ gst_thread_change_state (GstElement * element)
        * compare this block to the block
        */
 
-      
-      /* in glibc 2.2.5, pthread_attr_destroy does nothing more
-       * than return 0 */
-      if (pthread_attr_destroy (&thread->attr) != 0)
-	g_warning ("pthread_attr_destroy has failed !");
+      GST_DEBUG (GST_CAT_THREAD, "joining GThread %p", thread->thread_id);
+      g_thread_join (thread->thread_id);
 
-      GST_DEBUG (GST_CAT_THREAD, "joining pthread %ld", thread->thread_id);
-      if (pthread_join (thread->thread_id, NULL) != 0)
-        g_warning ("pthread_join has failed !\n");
-
-      thread->thread_id = -1;
-      
-      /* the stack was allocated when we created the thread
-       * using scheduler->get_preferred_stack */
-      if (thread->stack) {
-        GST_DEBUG (GST_CAT_THREAD, "freeing allocated stack (%p)", 
-	           thread->stack);
-        free (thread->stack);
-        thread->stack = NULL;
-      }
+      thread->thread_id = NULL;
+      thread->stack = NULL;
      
       THR_DEBUG ("unlocking mutex");
       g_mutex_unlock (thread->lock);
@@ -590,26 +557,14 @@ gst_thread_main_loop (void *arg)
 {
   GstThread *thread = NULL;
   gint stateset;
+  glong page_size;
+  gpointer stack_pointer;
+  gulong stack_offset;
 
   GST_DEBUG (GST_CAT_THREAD, "gst_thread_main_loop started");
   thread = GST_THREAD (arg);
   g_mutex_lock (thread->lock);
 
-  /* handle scheduler policy; do stuff if not the normal scheduler */
-  if (thread->sched_policy != SCHED_OTHER) {
-    struct sched_param sched_param;
-
-    memset (&sched_param, 0, sizeof (sched_param));
-    if (thread->priority == 0) {
-      thread->priority = sched_get_priority_max (thread->sched_policy);
-    }
-    sched_param.sched_priority = thread->priority;
-
-    if (sched_setscheduler (0, thread->sched_policy, &sched_param) != 0) {
-      GST_DEBUG (GST_CAT_THREAD, "not running with real-time priority");
-    }
-  }
-
   /* set up the element's scheduler */
   gst_scheduler_setup (GST_ELEMENT_SCHED (thread));
   GST_FLAG_UNSET (thread, GST_THREAD_STATE_REAPING);
@@ -617,6 +572,21 @@ gst_thread_main_loop (void *arg)
   thread->pid = getpid();
   THR_INFO_MAIN ("thread is running");
 
+  page_size = sysconf(_SC_PAGESIZE);
+  stack_pointer = (gpointer) &stack_pointer;
+
+  if(((gulong)stack_pointer & (page_size-1)) < (page_size>>1)){
+    /* stack grows up, I think */
+    /* FIXME this is probably not true for the main thread */
+    stack_offset = (gulong)stack_pointer & (page_size - 1);
+  }else{
+    /* stack grows down, I think */
+    stack_offset = STACK_SIZE - ((gulong)stack_pointer & (page_size - 1));
+  }
+  /* note the subtlety with pointer arithmetic */
+  thread->stack = stack_pointer - stack_offset;
+  thread->stack_size = STACK_SIZE;
+
   /* first we need to change the state of all the children */
   if (GST_ELEMENT_CLASS (parent_class)->change_state) {
     stateset = GST_ELEMENT_CLASS (parent_class)->change_state (GST_ELEMENT(thread));
@@ -647,7 +617,7 @@ gst_thread_main_loop (void *arg)
                         gst_element_state_get_name (GST_STATE_NULL),
                         gst_element_state_get_name (GST_STATE_PAUSED));
         g_cond_wait (thread->cond, thread->lock);
-	
+
 	/* this must have happened by a state change in the thread context */
 	if (GST_STATE_PENDING (thread) != GST_STATE_NULL &&
 	    GST_STATE_PENDING (thread) != GST_STATE_PAUSED) {
@@ -657,15 +627,15 @@ gst_thread_main_loop (void *arg)
 
         /* been signaled, we need to state transition now and signal back */
         gst_thread_update_state (thread);
-        THR_DEBUG_MAIN ("done with state transition, "
-	                "signaling back to parent process");
-        g_cond_signal (thread->cond);
         /* now we decide what to do next */
         if (GST_STATE (thread) == GST_STATE_NULL) {
           /* REAPING must be set, we can simply break this iteration */
           THR_DEBUG_MAIN ("set GST_THREAD_STATE_REAPING");
           GST_FLAG_SET (thread, GST_THREAD_STATE_REAPING);
 	}
+        THR_DEBUG_MAIN ("done with state transition, "
+	                "signaling back to parent process");
+        g_cond_signal (thread->cond);
         continue;
 
       case GST_STATE_PAUSED:
@@ -685,6 +655,7 @@ gst_thread_main_loop (void *arg)
 
         /* been signaled, we need to state transition now and signal back */
         gst_thread_update_state (thread);
+
         /* now we decide what to do next */
         if (GST_STATE (thread) != GST_STATE_PLAYING) {
           /* either READY or the state change failed for some reason */
@@ -707,7 +678,7 @@ gst_thread_main_loop (void *arg)
               GST_FLAG_UNSET (thread, GST_THREAD_STATE_SPINNING);
           }
 	  /* looks like we were stopped because of a statechange */
-	  if (GST_STATE_PENDING (thread)) {
+	  if (GST_STATE_PENDING (thread) != GST_STATE_VOID_PENDING) {
             gst_thread_update_state (thread);
 	  }
           /* once we're here, SPINNING has stopped, we should signal that we're done */
