@@ -109,7 +109,6 @@ static void 		gst_basic_scheduler_init 		(GstBasicScheduler * scheduler);
 static void 		gst_basic_scheduler_dispose 		(GObject *object);
 
 static void 		gst_basic_scheduler_setup 		(GstScheduler *sched);
-static gboolean 	gst_basic_scheduler_get_preferred_stack (GstScheduler *sched, gpointer *stack, gulong *size);
 static void 		gst_basic_scheduler_reset 		(GstScheduler *sched);
 static void		gst_basic_scheduler_add_element		(GstScheduler *sched, GstElement *element);
 static void     	gst_basic_scheduler_remove_element	(GstScheduler *sched, GstElement *element);
@@ -201,7 +200,6 @@ gst_basic_scheduler_class_init (GstBasicSchedulerClass * klass)
   gobject_class->dispose	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_dispose);
 
   gstscheduler_class->setup 		= GST_DEBUG_FUNCPTR (gst_basic_scheduler_setup);
-  gstscheduler_class->get_preferred_stack= GST_DEBUG_FUNCPTR (gst_basic_scheduler_get_preferred_stack);
   gstscheduler_class->reset	 	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_reset);
   gstscheduler_class->add_element 	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_add_element);
   gstscheduler_class->remove_element 	= GST_DEBUG_FUNCPTR (gst_basic_scheduler_remove_element);
@@ -978,18 +976,6 @@ gst_basic_scheduler_setup (GstScheduler *sched)
   }
 }
 
-static gboolean
-gst_basic_scheduler_get_preferred_stack (GstScheduler *sched, 
-                                         gpointer *stack, gulong *size)
-{
-  if (do_cothreads_stackquery (stack, size)) {
-    GST_DEBUG (GST_CAT_SCHEDULING, 
-	       "getting preferred stack size as %p and %lu", *stack, *size);
-    return TRUE;
-  }
-  return FALSE;
-}
-
 static void
 gst_basic_scheduler_reset (GstScheduler *sched)
 {
@@ -1012,9 +998,6 @@ gst_basic_scheduler_reset (GstScheduler *sched)
 static void
 gst_basic_scheduler_add_element (GstScheduler * sched, GstElement * element)
 {
-  GList *pads;
-  GstPad *pad;
-  GstElement *peerelement;
   GstSchedulerChain *chain;
   GstBasicScheduler *bsched = GST_BASIC_SCHEDULER (sched);
 
@@ -1032,27 +1015,6 @@ gst_basic_scheduler_add_element (GstScheduler * sched, GstElement * element)
   /* create a chain to hold it, and add */
   chain = gst_basic_scheduler_chain_new (bsched);
   gst_basic_scheduler_chain_add_element (chain, element);
-
-  /* set the sched pointer in all the pads */
-  pads = element->pads;
-  while (pads) {
-    pad = GST_PAD (pads->data);
-    pads = g_list_next (pads);
-
-    /* we only operate on real pads */
-    if (!GST_IS_REAL_PAD (pad))
-      continue;
-
-    /* if the peer element exists and is a candidate */
-    if (GST_PAD_PEER (pad)) {
-      peerelement = GST_PAD_PARENT (GST_PAD_PEER (pad));
-      if (GST_ELEMENT_SCHED (element) == GST_ELEMENT_SCHED (peerelement)) {
-	GST_INFO (GST_CAT_SCHEDULING, "peer is in same scheduler, chaining together");
-	/* make sure that the two elements are in the same chain */
-	gst_basic_scheduler_chain_elements (bsched, element, peerelement);
-      }
-    }
-  }
 }
 
 static void
@@ -1296,7 +1258,11 @@ static GstClockReturn
 gst_basic_scheduler_clock_wait (GstScheduler *sched, GstElement *element,
 				GstClock *clock, GstClockTime time, GstClockTimeDiff *jitter)
 {
-  return gst_clock_wait (clock, time, jitter);
+  GstClockID id;
+  
+  id = gst_clock_new_single_shot_id (clock, time);
+
+  return gst_clock_id_wait (id, jitter);
 }
 
 static GstSchedulerState
